### ms-gkdi.idl
"""
RPC definitions for the following interfaces:
- ISDKey (v1.0): b9785960-524f-11df-8b6d-83dcded72085
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    register_dcerpc_interface,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class GetKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("cbTargetSD", None, size_of="pbTargetSD"),
        NDRConfStrLenField("pbTargetSD", "", size_is=lambda pkt: pkt.cbTargetSD),
        NDRFullPointerField(NDRPacketField("pRootKeyID", GUID(), GUID)),
        NDRSignedIntField("L0KeyID", 0),
        NDRSignedIntField("L1KeyID", 0),
        NDRSignedIntField("L2KeyID", 0),
    ]


class GetKey_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcbOut", None, size_of="ppbOut"),
        NDRConfStrLenField("ppbOut", "", size_is=lambda pkt: pkt.pcbOut),
        NDRIntField("status", 0),
    ]


ISDKEY_OPNUMS = {0: DceRpcOp(GetKey_Request, GetKey_Response)}
register_dcerpc_interface(
    name="ISDKey",
    uuid=uuid.UUID("b9785960-524f-11df-8b6d-83dcded72085"),
    version="1.0",
    opnums=ISDKEY_OPNUMS,
)



### ms-com.idl
"""
RPC definitions for the following interfaces:
- ITransactionStream (v0.0): 97199110-DB2E-11d1-A251-0000F805CA53
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_com_interface,
)


class GetSeqAndTxViaExport_Request(NDRPacket):
    fields_desc = [
        NDRIntField("ulKnownSeq", 0),
        NDRIntField("ulcbWhereabouts", None, size_of="rgbWhereabouts"),
        NDRConfStrLenField(
            "rgbWhereabouts", "", size_is=lambda pkt: pkt.ulcbWhereabouts
        ),
    ]


class GetSeqAndTxViaExport_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pulCurrentSeq", 0),
        NDRIntField("pulcbExportCookie", None, size_of="prgbExportCookie"),
        NDRConfStrLenField(
            "prgbExportCookie", "", size_is=lambda pkt: pkt.pulcbExportCookie
        ),
        NDRIntField("status", 0),
    ]


class GetSeqAndTxViaTransmitter_Request(NDRPacket):
    fields_desc = [NDRIntField("ulKnownSeq", 0)]


class GetSeqAndTxViaTransmitter_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pulCurrentSeq", 0),
        NDRIntField("pulcbTransmitterBuffer", None, size_of="prgbTransmitterBuffer"),
        NDRConfStrLenField(
            "prgbTransmitterBuffer", "", size_is=lambda pkt: pkt.pulcbTransmitterBuffer
        ),
        NDRIntField("status", 0),
    ]


class GetTxViaExport_Request(NDRPacket):
    fields_desc = [
        NDRIntField("ulRequestSeq", 0),
        NDRIntField("ulcbWhereabouts", None, size_of="rgbWhereabouts"),
        NDRConfStrLenField(
            "rgbWhereabouts", "", size_is=lambda pkt: pkt.ulcbWhereabouts
        ),
    ]


class GetTxViaExport_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pulcbExportCookie", None, size_of="prgbExportCookie"),
        NDRConfStrLenField(
            "prgbExportCookie", "", size_is=lambda pkt: pkt.pulcbExportCookie
        ),
        NDRIntField("status", 0),
    ]


class GetTxViaTransmitter_Request(NDRPacket):
    fields_desc = [NDRIntField("ulRequestSeq", 0)]


class GetTxViaTransmitter_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pulcbTransmitterBuffer", None, size_of="prgbTransmitterBuffer"),
        NDRConfStrLenField(
            "prgbTransmitterBuffer", "", size_is=lambda pkt: pkt.pulcbTransmitterBuffer
        ),
        NDRIntField("status", 0),
    ]


ITRANSACTIONSTREAM_OPNUMS = {
    0: DceRpcOp(GetSeqAndTxViaExport_Request, GetSeqAndTxViaExport_Response),
    1: DceRpcOp(GetSeqAndTxViaTransmitter_Request, GetSeqAndTxViaTransmitter_Response),
    2: DceRpcOp(GetTxViaExport_Request, GetTxViaExport_Response),
    3: DceRpcOp(GetTxViaTransmitter_Request, GetTxViaTransmitter_Response),
}
register_com_interface(
    name="ITransactionStream",
    uuid=uuid.UUID("97199110-DB2E-11d1-A251-0000F805CA53"),
    opnums=ITRANSACTIONSTREAM_OPNUMS,
)



### ms-mqrr.idl
"""
RPC definitions for the following interfaces:
- RemoteRead (v1.0): 1a9134dd-7b39-45ba-ad88-44d01ca47f28
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRUnionField,
    register_dcerpc_interface,
)


class R_GetServerPort_Request(NDRPacket):
    fields_desc = []


class R_GetServerPort_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class QUEUE_FORMAT_TYPE(IntEnum):
    QUEUE_FORMAT_TYPE_UNKNOWN = 0
    QUEUE_FORMAT_TYPE_PUBLIC = 1
    QUEUE_FORMAT_TYPE_PRIVATE = 2
    QUEUE_FORMAT_TYPE_DIRECT = 3
    QUEUE_FORMAT_TYPE_MACHINE = 4
    QUEUE_FORMAT_TYPE_CONNECTOR = 5
    QUEUE_FORMAT_TYPE_DL = 6
    QUEUE_FORMAT_TYPE_MULTICAST = 7
    QUEUE_FORMAT_TYPE_SUBQUEUE = 8


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class OBJECTID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Lineage", GUID(), GUID),
        NDRIntField("Uniquifier", 0),
    ]


class DL_ID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("m_DlGuid", GUID(), GUID),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("m_pwzDomain", ""), deferred=True
        ),
    ]


class MULTICAST_ID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("m_address", 0), NDRIntField("m_port", 0)]


class QUEUE_FORMAT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("m_qft", 0),
        NDRByteField("m_SuffixAndFlags", 0),
        NDRShortField("m_reserved", 0),
        NDRUnionField(
            [
                (
                    StrFixedLenField("value", "", length=0),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", OBJECTID(), OBJECTID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", DL_ID(), DL_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", MULTICAST_ID(), MULTICAST_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(1, 8),
            switch_fmt=("B", "B"),
        ),
    ]


class R_OpenQueue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
        NDRIntField("dwAccess", 0),
        NDRIntField("dwShareMode", 0),
        NDRPacketField("pClientId", GUID(), GUID),
        NDRSignedIntField("fNonRoutingServer", 0),
        NDRByteField("Major", 0),
        NDRByteField("Minor", 0),
        NDRShortField("BuildNumber", 0),
        NDRSignedIntField("fWorkgroup", 0),
    ]


class R_OpenQueue_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class R_CloseQueue_Request(NDRPacket):
    fields_desc = [NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle)]


class R_CloseQueue_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class R_CreateCursor_Request(NDRPacket):
    fields_desc = [NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle)]


class R_CreateCursor_Response(NDRPacket):
    fields_desc = [NDRIntField("phCursor", 0), NDRIntField("status", 0)]


class R_CloseCursor_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("hCursor", 0),
    ]


class R_CloseCursor_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_PurgeQueue_Request(NDRPacket):
    fields_desc = [NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle)]


class R_PurgeQueue_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SectionType(IntEnum):
    stFullPacket = 0
    stBinaryFirstSection = 1
    stBinarySecondSection = 2
    stSrmpFirstSection = 3
    stSrmpSecondSection = 4


class SectionBuffer(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("SectionBufferType", 0, SectionType),
        NDRIntField("SectionSizeAlloc", 0),
        NDRIntField("SectionSize", None, size_of="pSectionBuffer"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pSectionBuffer", "", size_is=lambda pkt: pkt.SectionSize
            ),
            deferred=True,
        ),
    ]


class R_StartReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRLongField("LookupId", 0),
        NDRIntField("hCursor", 0),
        NDRIntField("ulAction", 0),
        NDRIntField("ulTimeout", 0),
        NDRIntField("dwRequestId", 0),
        NDRIntField("dwMaxBodySize", 0),
        NDRIntField("dwMaxCompoundMessageSize", 0),
    ]


class R_StartReceive_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwArriveTime", 0),
        NDRLongField("pSequenceId", 0),
        NDRIntField("pdwNumberOfSections", None, size_of="ppPacketSections"),
        NDRConfPacketListField(
            "ppPacketSections",
            [],
            SectionBuffer,
            size_is=lambda pkt: pkt.pdwNumberOfSections,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class R_CancelReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwRequestId", 0),
    ]


class R_CancelReceive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_EndReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwAck", 0),
        NDRIntField("dwRequestId", 0),
    ]


class R_EndReceive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class XACTUOW(NDRPacket):
    fields_desc = [StrFixedLenField("rgb", "", length=16)]


class R_MoveMessage_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContextFrom", NDRContextHandle(), NDRContextHandle),
        NDRLongField("ullContextTo", 0),
        NDRLongField("LookupId", 0),
        NDRPacketField("pTransactionId", XACTUOW(), XACTUOW),
    ]


class R_MoveMessage_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_OpenQueueForMove_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
        NDRIntField("dwAccess", 0),
        NDRIntField("dwShareMode", 0),
        NDRPacketField("pClientId", GUID(), GUID),
        NDRSignedIntField("fNonRoutingServer", 0),
        NDRByteField("Major", 0),
        NDRByteField("Minor", 0),
        NDRShortField("BuildNumber", 0),
        NDRSignedIntField("fWorkgroup", 0),
    ]


class R_OpenQueueForMove_Response(NDRPacket):
    fields_desc = [
        NDRLongField("pMoveContext", 0),
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class R_QMEnlistRemoteTransaction_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pTransactionId", XACTUOW(), XACTUOW),
        NDRIntField("cbPropagationToken", None, size_of="pbPropagationToken"),
        NDRConfStrLenField(
            "pbPropagationToken", "", size_is=lambda pkt: pkt.cbPropagationToken
        ),
        NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
    ]


class R_QMEnlistRemoteTransaction_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_StartTransactionalReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRLongField("LookupId", 0),
        NDRIntField("hCursor", 0),
        NDRIntField("ulAction", 0),
        NDRIntField("ulTimeout", 0),
        NDRIntField("dwRequestId", 0),
        NDRIntField("dwMaxBodySize", 0),
        NDRIntField("dwMaxCompoundMessageSize", 0),
        NDRPacketField("pTransactionId", XACTUOW(), XACTUOW),
    ]


class R_StartTransactionalReceive_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwArriveTime", 0),
        NDRLongField("pSequenceId", 0),
        NDRIntField("pdwNumberOfSections", None, size_of="ppPacketSections"),
        NDRConfPacketListField(
            "ppPacketSections",
            [],
            SectionBuffer,
            size_is=lambda pkt: pkt.pdwNumberOfSections,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class R_SetUserAcknowledgementClass_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRLongField("LookupId", 0),
        NDRShortField("usClass", 0),
    ]


class R_SetUserAcknowledgementClass_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_EndTransactionalReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwAck", 0),
        NDRIntField("dwRequestId", 0),
    ]


class R_EndTransactionalReceive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


REMOTEREAD_OPNUMS = {
    0: DceRpcOp(R_GetServerPort_Request, R_GetServerPort_Response),
    # 1: Opnum1NotUsedOnWire,
    2: DceRpcOp(R_OpenQueue_Request, R_OpenQueue_Response),
    3: DceRpcOp(R_CloseQueue_Request, R_CloseQueue_Response),
    4: DceRpcOp(R_CreateCursor_Request, R_CreateCursor_Response),
    5: DceRpcOp(R_CloseCursor_Request, R_CloseCursor_Response),
    6: DceRpcOp(R_PurgeQueue_Request, R_PurgeQueue_Response),
    7: DceRpcOp(R_StartReceive_Request, R_StartReceive_Response),
    8: DceRpcOp(R_CancelReceive_Request, R_CancelReceive_Response),
    9: DceRpcOp(R_EndReceive_Request, R_EndReceive_Response),
    10: DceRpcOp(R_MoveMessage_Request, R_MoveMessage_Response),
    11: DceRpcOp(R_OpenQueueForMove_Request, R_OpenQueueForMove_Response),
    12: DceRpcOp(
        R_QMEnlistRemoteTransaction_Request, R_QMEnlistRemoteTransaction_Response
    ),
    13: DceRpcOp(
        R_StartTransactionalReceive_Request, R_StartTransactionalReceive_Response
    ),
    14: DceRpcOp(
        R_SetUserAcknowledgementClass_Request, R_SetUserAcknowledgementClass_Response
    ),
    15: DceRpcOp(R_EndTransactionalReceive_Request, R_EndTransactionalReceive_Response),
}
register_dcerpc_interface(
    name="RemoteRead",
    uuid=uuid.UUID("1a9134dd-7b39-45ba-ad88-44d01ca47f28"),
    version="1.0",
    opnums=REMOTEREAD_OPNUMS,
)



### ms-oxnspi.idl
"""
RPC definitions for the following interfaces:
- nspi (v56.0): F5CC5A18-4264-101A-8C59-08002B2F8426
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarFieldListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRRecursiveField,
    NDRShortField,
    NDRSignedByteField,
    NDRSignedIntField,
    NDRSignedShortField,
    NDRUnionField,
    register_dcerpc_interface,
)


class STAT(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("SortType", 0),
        NDRIntField("ContainerID", 0),
        NDRIntField("CurrentRec", 0),
        NDRSignedIntField("Delta", 0),
        NDRIntField("NumPos", 0),
        NDRIntField("TotalRecs", 0),
        NDRIntField("CodePage", 0),
        NDRIntField("TemplateLocale", 0),
        NDRIntField("SortLocale", 0),
    ]


class FlatUID_r(NDRPacket):
    fields_desc = [StrFixedLenField("ab", "", length=16)]


class NspiBind_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwFlags", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(NDRPacketField("pServerGuid", FlatUID_r(), FlatUID_r)),
    ]


class NspiBind_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pServerGuid", FlatUID_r(), FlatUID_r)),
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class NspiUnbind_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
    ]


class NspiUnbind_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class NspiUpdateStat_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(NDRSignedIntField("plDelta", 0)),
    ]


class NspiUpdateStat_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(NDRSignedIntField("plDelta", 0)),
        NDRIntField("status", 0),
    ]


class PropertyTagArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["aulPropTag"]
    fields_desc = [
        NDRIntField("cValues", None, size_of="aulPropTag"),
        NDRConfVarFieldListField(
            "aulPropTag",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: (pkt.cValues + 1),
            length_is=lambda pkt: pkt.cValues,
            conformant_in_struct=True,
        ),
    ]


class Binary_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cb", None, size_of="lpb"),
        NDRFullPointerField(
            NDRConfStrLenField("lpb", "", size_is=lambda pkt: pkt.cb), deferred=True
        ),
    ]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class ShortArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lpi"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("lpi", "", size_is=lambda pkt: pkt.cValues),
            deferred=True,
        ),
    ]


class LongArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lpl"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "lpl", [], NDRSignedIntField("lpl", 0), size_is=lambda pkt: pkt.cValues
            ),
            deferred=True,
        ),
    ]


class StringArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lppszA"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "lppszA",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullField("lppszA", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.cValues,
            ),
            deferred=True,
        ),
    ]


class BinaryArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lpbin"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "lpbin", [], Binary_r, size_is=lambda pkt: pkt.cValues
            ),
            deferred=True,
        ),
    ]


class FlatUIDArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lpguid"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "lpguid", [], FlatUID_r, size_is=lambda pkt: pkt.cValues, ptr_pack=True
            ),
            deferred=True,
        ),
    ]


class WStringArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lppszW"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "lppszW",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("lppszW", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.cValues,
            ),
            deferred=True,
        ),
    ]


class DateTimeArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cValues", None, size_of="lpft"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "lpft", [], FILETIME, size_is=lambda pkt: pkt.cValues
            ),
            deferred=True,
        ),
    ]


class _PropertyValue_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulPropTag", 0),
        NDRIntField("ulReserved", 0),
        NDRUnionField(
            [
                (
                    NDRSignedShortField("Value", 0),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 2
                        ),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRSignedIntField("Value", 0),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 3
                        ),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
                (
                    NDRShortField("Value", 0),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 11
                        ),
                        (lambda _, val: val.tag == 11),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullField("Value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 30
                        ),
                        (lambda _, val: val.tag == 30),
                    ),
                ),
                (
                    NDRPacketField("Value", Binary_r(), Binary_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 258
                        ),
                        (lambda _, val: val.tag == 258),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("Value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 31
                        ),
                        (lambda _, val: val.tag == 31),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("Value", FlatUID_r(), FlatUID_r), deferred=True
                    ),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 72
                        ),
                        (lambda _, val: val.tag == 72),
                    ),
                ),
                (
                    NDRPacketField("Value", FILETIME(), FILETIME),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 64
                        ),
                        (lambda _, val: val.tag == 64),
                    ),
                ),
                (
                    NDRSignedIntField("Value", 0),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 10
                        ),
                        (lambda _, val: val.tag == 10),
                    ),
                ),
                (
                    NDRPacketField("Value", ShortArray_r(), ShortArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4098
                        ),
                        (lambda _, val: val.tag == 4098),
                    ),
                ),
                (
                    NDRPacketField("Value", LongArray_r(), LongArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4099
                        ),
                        (lambda _, val: val.tag == 4099),
                    ),
                ),
                (
                    NDRPacketField("Value", StringArray_r(), StringArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4126
                        ),
                        (lambda _, val: val.tag == 4126),
                    ),
                ),
                (
                    NDRPacketField("Value", BinaryArray_r(), BinaryArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4354
                        ),
                        (lambda _, val: val.tag == 4354),
                    ),
                ),
                (
                    NDRPacketField("Value", FlatUIDArray_r(), FlatUIDArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4168
                        ),
                        (lambda _, val: val.tag == 4168),
                    ),
                ),
                (
                    NDRPacketField("Value", WStringArray_r(), WStringArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4127
                        ),
                        (lambda _, val: val.tag == 4127),
                    ),
                ),
                (
                    NDRPacketField("Value", DateTimeArray_r(), DateTimeArray_r),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            == 4160
                        ),
                        (lambda _, val: val.tag == 4160),
                    ),
                ),
                (
                    NDRSignedIntField("Value", 0),
                    (
                        (
                            lambda pkt: (
                                (getattr(pkt, "ulPropTag", None) & 65535)
                                & 0xFFFFFFFFFFFFFFFF
                            )
                            in [1, 13]
                        ),
                        (lambda _, val: val.tag in [1, 13]),
                    ),
                ),
            ],
            StrFixedLenField("Value", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class PropertyRow_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Reserved", 0),
        NDRIntField("cValues", None, size_of="lpProps"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "lpProps", [], _PropertyValue_r, size_is=lambda pkt: pkt.cValues
            ),
            deferred=True,
        ),
    ]


class PropertyRowSet_r(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["aRow"]
    fields_desc = [
        NDRIntField("cRows", None, size_of="aRow"),
        NDRConfPacketListField(
            "aRow",
            [],
            PropertyRow_r,
            size_is=lambda pkt: pkt.cRows,
            conformant_in_struct=True,
        ),
    ]


class NspiQueryRows_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRIntField("dwETableCount", None, size_of="lpETable"),
        NDRConfFieldListField(
            "lpETable", [], NDRIntField("", 0), size_is=lambda pkt: pkt.dwETableCount
        ),
        NDRIntField("Count", 0),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
    ]


class NspiQueryRows_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiSeekEntries_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRPacketField("pTarget", _PropertyValue_r(), _PropertyValue_r),
        NDRFullPointerField(
            NDRPacketField("lpETable", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
    ]


class NspiSeekEntries_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


class _Restriction_r(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("rt", 0), NDRRecursiveField("res")]


class PropertyName_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("lpguid", FlatUID_r(), FlatUID_r), deferred=True
        ),
        NDRIntField("ulReserved", 0),
        NDRSignedIntField("lID", 0),
    ]


class NspiGetMatches_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved1", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("pReserved", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRIntField("Reserved2", 0),
        NDRFullPointerField(NDRPacketField("Filter", _Restriction_r(), _Restriction_r)),
        NDRFullPointerField(
            NDRPacketField("lpPropName", PropertyName_r(), PropertyName_r)
        ),
        NDRIntField("ulRequested", 0),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
    ]


class NspiGetMatches_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("ppOutMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiResortRestriction_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRPacketField("pInMIds", PropertyTagArray_r(), PropertyTagArray_r),
        NDRFullPointerField(
            NDRPacketField("ppOutMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
    ]


class NspiResortRestriction_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("ppOutMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRIntField("status", 0),
    ]


class StringsArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Count", None, size_of="Strings"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "Strings",
                [],
                NDRFullPointerField(NDRSignedByteField("Strings", 0), deferred=True),
                size_is=lambda pkt: pkt.Count,
            ),
            deferred=True,
        ),
    ]


class NspiDNToMId_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pNames", StringsArray_r(), StringsArray_r),
    ]


class NspiDNToMId_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppOutMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiGetPropList_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRIntField("dwMId", 0),
        NDRIntField("CodePage", 0),
    ]


class NspiGetPropList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiGetProps_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
    ]


class NspiGetProps_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppRows", PropertyRow_r(), PropertyRow_r)),
        NDRIntField("status", 0),
    ]


class NspiCompareMIds_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRIntField("MId1", 0),
        NDRIntField("MId2", 0),
    ]


class NspiCompareMIds_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("plResult", 0), NDRIntField("status", 0)]


class NspiModProps_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRPacketField("pRow", PropertyRow_r(), PropertyRow_r),
    ]


class NspiModProps_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NspiGetSpecialTable_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRIntField("lpVersion", 0),
    ]


class NspiGetSpecialTable_Response(NDRPacket):
    fields_desc = [
        NDRIntField("lpVersion", 0),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiGetTemplateInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRIntField("ulType", 0),
        NDRFullPointerField(NDRConfVarStrNullField("pDN", "")),
        NDRIntField("dwCodePage", 0),
        NDRIntField("dwLocaleID", 0),
    ]


class NspiGetTemplateInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppData", PropertyRow_r(), PropertyRow_r)),
        NDRIntField("status", 0),
    ]


class NspiModLinkAtt_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFlags", 0),
        NDRIntField("ulPropTag", 0),
        NDRIntField("dwMId", 0),
        NDRPacketField("lpEntryIds", BinaryArray_r(), BinaryArray_r),
    ]


class NspiModLinkAtt_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NspiQueryColumns_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRIntField("dwFlags", 0),
    ]


class NspiQueryColumns_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppColumns", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRIntField("status", 0),
    ]


class NspiResolveNames_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRPacketField("paStr", StringsArray_r(), StringsArray_r),
    ]


class NspiResolveNames_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


class WStringsArray_r(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Count", None, size_of="Strings"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "Strings",
                [],
                NDRFullPointerField(NDRShortField("Strings", 0), deferred=True),
                size_is=lambda pkt: pkt.Count,
            ),
            deferred=True,
        ),
    ]


class NspiResolveNamesW_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hRpc", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Reserved", 0),
        NDRPacketField("pStat", STAT(), STAT),
        NDRFullPointerField(
            NDRPacketField("pPropTags", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRPacketField("paWStr", WStringsArray_r(), WStringsArray_r),
    ]


class NspiResolveNamesW_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppMIds", PropertyTagArray_r(), PropertyTagArray_r)
        ),
        NDRFullPointerField(
            NDRPacketField("ppRows", PropertyRowSet_r(), PropertyRowSet_r)
        ),
        NDRIntField("status", 0),
    ]


NSPI_OPNUMS = {
    0: DceRpcOp(NspiBind_Request, NspiBind_Response),
    1: DceRpcOp(NspiUnbind_Request, NspiUnbind_Response),
    2: DceRpcOp(NspiUpdateStat_Request, NspiUpdateStat_Response),
    3: DceRpcOp(NspiQueryRows_Request, NspiQueryRows_Response),
    4: DceRpcOp(NspiSeekEntries_Request, NspiSeekEntries_Response),
    5: DceRpcOp(NspiGetMatches_Request, NspiGetMatches_Response),
    6: DceRpcOp(NspiResortRestriction_Request, NspiResortRestriction_Response),
    7: DceRpcOp(NspiDNToMId_Request, NspiDNToMId_Response),
    8: DceRpcOp(NspiGetPropList_Request, NspiGetPropList_Response),
    9: DceRpcOp(NspiGetProps_Request, NspiGetProps_Response),
    10: DceRpcOp(NspiCompareMIds_Request, NspiCompareMIds_Response),
    11: DceRpcOp(NspiModProps_Request, NspiModProps_Response),
    12: DceRpcOp(NspiGetSpecialTable_Request, NspiGetSpecialTable_Response),
    13: DceRpcOp(NspiGetTemplateInfo_Request, NspiGetTemplateInfo_Response),
    14: DceRpcOp(NspiModLinkAtt_Request, NspiModLinkAtt_Response),
    # 15: Opnum15NotUsedOnWire,
    16: DceRpcOp(NspiQueryColumns_Request, NspiQueryColumns_Response),
    # 17: Opnum17NotUsedOnWire,
    # 18: Opnum18NotUsedOnWire,
    19: DceRpcOp(NspiResolveNames_Request, NspiResolveNames_Response),
    20: DceRpcOp(NspiResolveNamesW_Request, NspiResolveNamesW_Response),
}
register_dcerpc_interface(
    name="nspi",
    uuid=uuid.UUID("F5CC5A18-4264-101A-8C59-08002B2F8426"),
    version="56.0",
    opnums=NSPI_OPNUMS,
)



### ms-swn.idl
"""
RPC definitions for the following interfaces:
- Witness (v1.1): ccd8c074-d0e5-4a40-92b4-d074faa6ba28
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class PWITNESS_INTERFACE_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        StrFixedLenFieldUtf16("InterfaceGroupName", "", length=260 * 2),
        NDRIntField("Version", 0),
        NDRShortField("State", 0),
        NDRIntField("IPV4", 0),
        StrFixedLenFieldUtf16("IPV6", "", length=8 * 2),
        NDRIntField("Flags", 0),
    ]


class PWITNESS_INTERFACE_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("NumberOfInterfaces", None, size_of="InterfaceInfo"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "InterfaceInfo",
                [PWITNESS_INTERFACE_INFO()],
                PWITNESS_INTERFACE_INFO,
                size_is=lambda pkt: pkt.NumberOfInterfaces,
            ),
            deferred=True,
        ),
    ]


class WitnessrGetInterfaceList_Request(NDRPacket):
    fields_desc = []


class WitnessrGetInterfaceList_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "InterfaceList", PWITNESS_INTERFACE_LIST(), PWITNESS_INTERFACE_LIST
            )
        ),
        NDRIntField("status", 0),
    ]


class WitnessrRegister_Request(NDRPacket):
    fields_desc = [
        NDRIntField("Version", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NetName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("IpAddress", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ClientComputerName", "")),
    ]


class WitnessrRegister_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ppContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class WitnessrUnRegister_Request(NDRPacket):
    fields_desc = [NDRPacketField("pContext", NDRContextHandle(), NDRContextHandle)]


class WitnessrUnRegister_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PRESP_ASYNC_NOTIFY(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("MessageType", 0),
        NDRIntField("Length", None, size_of="MessageBuffer"),
        NDRIntField("NumberOfMessages", 0),
        NDRFullPointerField(
            NDRConfStrLenField("MessageBuffer", "", size_is=lambda pkt: pkt.Length),
            deferred=True,
        ),
    ]


class WitnessrAsyncNotify_Request(NDRPacket):
    fields_desc = [NDRPacketField("pContext", NDRContextHandle(), NDRContextHandle)]


class WitnessrAsyncNotify_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pResp", PRESP_ASYNC_NOTIFY(), PRESP_ASYNC_NOTIFY)
        ),
        NDRIntField("status", 0),
    ]


class WitnessrRegisterEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("Version", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NetName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ShareName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("IpAddress", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ClientComputerName", "")),
        NDRIntField("Flags", 0),
        NDRIntField("KeepAliveTimeout", 0),
    ]


class WitnessrRegisterEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ppContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


WITNESS_OPNUMS = {
    0: DceRpcOp(WitnessrGetInterfaceList_Request, WitnessrGetInterfaceList_Response),
    1: DceRpcOp(WitnessrRegister_Request, WitnessrRegister_Response),
    2: DceRpcOp(WitnessrUnRegister_Request, WitnessrUnRegister_Response),
    3: DceRpcOp(WitnessrAsyncNotify_Request, WitnessrAsyncNotify_Response),
    4: DceRpcOp(WitnessrRegisterEx_Request, WitnessrRegisterEx_Response),
}
register_dcerpc_interface(
    name="Witness",
    uuid=uuid.UUID("ccd8c074-d0e5-4a40-92b4-d074faa6ba28"),
    version="1.1",
    opnums=WITNESS_OPNUMS,
)



### ms-tsts_tspubrpc.idl
"""
RPC definitions for the following interfaces:
- TermSrvSession (v1.0): 484809d6-4239-471b-b5bc-61df8c23ac48
- TermSrvNotification (v1.0): 11899a43-2b68-4a76-92e3-a3d6ad8c26ce
- TermSrvEnumeration (v1.0): 88143fd0-c28d-4b2b-8fef-8d882f6a9390
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_dcerpc_interface,
)


class RpcOpenSession_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("SessionId", 0)]


class RpcOpenSession_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phSession", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RpcCloseSession_Request(NDRPacket):
    fields_desc = [NDRPacketField("phSession", NDRContextHandle(), NDRContextHandle)]


class RpcCloseSession_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phSession", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RpcConnect_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle),
        NDRSignedIntField("TargetSessionId", 0),
        NDRConfVarStrNullFieldUtf16("szPassword", ""),
    ]


class RpcConnect_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcDisconnect_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcDisconnect_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcLogoff_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcLogoff_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcGetUserName_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcGetUserName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszUserName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszDomain", "")),
        NDRIntField("status", 0),
    ]


class RpcGetTerminalName_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcGetTerminalName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszTerminalName", "")),
        NDRIntField("status", 0),
    ]


class RpcGetState_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcGetState_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("plState", 0), NDRIntField("status", 0)]


class RpcIsSessionDesktopLocked_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcIsSessionDesktopLocked_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcShowMessageBox_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("szTitle", ""),
        NDRConfVarStrNullFieldUtf16("szMessage", ""),
        NDRIntField("ulStyle", 0),
        NDRIntField("ulTimeout", 0),
        NDRSignedIntField("bDoNotWait", 0),
    ]


class RpcShowMessageBox_Response(NDRPacket):
    fields_desc = [NDRIntField("pulResponse", 0), NDRIntField("status", 0)]


class RpcGetTimes_Request(NDRPacket):
    fields_desc = [NDRPacketField("hSession", NDRContextHandle(), NDRContextHandle)]


class RpcGetTimes_Response(NDRPacket):
    fields_desc = [
        NDRSignedLongField("pConnectTime", 0),
        NDRSignedLongField("pDisconnectTime", 0),
        NDRSignedLongField("pLogonTime", 0),
        NDRIntField("status", 0),
    ]


class TS_COUNTER_HEADER(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwCounterID", 0), NDRIntField("bResult", 0)]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class PTS_COUNTER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("counterHead", TS_COUNTER_HEADER(), TS_COUNTER_HEADER),
        NDRIntField("dwValue", 0),
        NDRPacketField("startTime", LARGE_INTEGER(), LARGE_INTEGER),
    ]


class RpcGetSessionCounters_Request(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "pCounter", [PTS_COUNTER()], PTS_COUNTER, size_is=lambda pkt: pkt.uEntries
        ),
        NDRIntField("uEntries", None, size_of="pCounter"),
    ]


class RpcGetSessionCounters_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "pCounter", [PTS_COUNTER()], PTS_COUNTER, size_is=lambda pkt: pkt.uEntries
        ),
        NDRIntField("status", 0),
    ]


class PLSMSESSIONINFORMATION(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszUserName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszDomain", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszTerminalName", ""), deferred=True
        ),
        NDRSignedIntField("SessionState", 0),
        NDRSignedIntField("DesktopLocked", 0),
        NDRSignedLongField("ConnectTime", 0),
        NDRSignedLongField("DisconnectTime", 0),
        NDRSignedLongField("LogonTime", 0),
    ]


class RpcGetSessionInformation_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("SessionId", 0)]


class RpcGetSessionInformation_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "pSessionInfo", PLSMSESSIONINFORMATION(), PLSMSESSIONINFORMATION
        ),
        NDRIntField("status", 0),
    ]


class RpcGetLoggedOnCount_Request(NDRPacket):
    fields_desc = []


class RpcGetLoggedOnCount_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pUserSessions", 0),
        NDRIntField("pDeviceSessions", 0),
        NDRIntField("status", 0),
    ]


class RpcGetSessionType_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("SessionId", 0)]


class RpcGetSessionType_Response(NDRPacket):
    fields_desc = [NDRIntField("pSessionType", 0), NDRIntField("status", 0)]


class LSM_SESSIONINFO_EX_LEVEL1(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedIntField("SessionState", 0),
        NDRSignedIntField("SessionFlags", 0),
        StrFixedLenFieldUtf16("SessionName", "", length=33 * 2),
        StrFixedLenFieldUtf16("DomainName", "", length=18 * 2),
        StrFixedLenFieldUtf16("UserName", "", length=21 * 2),
        NDRSignedLongField("ConnectTime", 0),
        NDRSignedLongField("DisconnectTime", 0),
        NDRSignedLongField("LogonTime", 0),
        NDRSignedLongField("LastInputTime", 0),
        NDRIntField("ProtocolDataSize", None, size_of="ProtocolData"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "ProtocolData", "", size_is=lambda pkt: pkt.ProtocolDataSize
            ),
            deferred=True,
        ),
    ]


class PLSMSESSIONINFORMATION_EX(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "Data", LSM_SESSIONINFO_EX_LEVEL1(), LSM_SESSIONINFO_EX_LEVEL1
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                )
            ],
            StrFixedLenField("Data", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class RpcGetSessionInformationEx_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("SessionId", 0), NDRIntField("Level", 0)]


class RpcGetSessionInformationEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "LSMSessionInfoExPtr",
            PLSMSESSIONINFORMATION_EX(),
            PLSMSESSIONINFORMATION_EX,
        ),
        NDRIntField("status", 0),
    ]


TERMSRVSESSION_OPNUMS = {
    0: DceRpcOp(RpcOpenSession_Request, RpcOpenSession_Response),
    1: DceRpcOp(RpcCloseSession_Request, RpcCloseSession_Response),
    2: DceRpcOp(RpcConnect_Request, RpcConnect_Response),
    3: DceRpcOp(RpcDisconnect_Request, RpcDisconnect_Response),
    4: DceRpcOp(RpcLogoff_Request, RpcLogoff_Response),
    5: DceRpcOp(RpcGetUserName_Request, RpcGetUserName_Response),
    6: DceRpcOp(RpcGetTerminalName_Request, RpcGetTerminalName_Response),
    7: DceRpcOp(RpcGetState_Request, RpcGetState_Response),
    8: DceRpcOp(RpcIsSessionDesktopLocked_Request, RpcIsSessionDesktopLocked_Response),
    9: DceRpcOp(RpcShowMessageBox_Request, RpcShowMessageBox_Response),
    10: DceRpcOp(RpcGetTimes_Request, RpcGetTimes_Response),
    11: DceRpcOp(RpcGetSessionCounters_Request, RpcGetSessionCounters_Response),
    12: DceRpcOp(RpcGetSessionInformation_Request, RpcGetSessionInformation_Response),
    # 13: Opnum13NotUsedOnWire,
    # 14: Opnum14NotUsedOnWire,
    15: DceRpcOp(RpcGetLoggedOnCount_Request, RpcGetLoggedOnCount_Response),
    16: DceRpcOp(RpcGetSessionType_Request, RpcGetSessionType_Response),
    17: DceRpcOp(
        RpcGetSessionInformationEx_Request, RpcGetSessionInformationEx_Response
    ),
    # 18: Opnum18NotUsedOnWire,
    # 19: Opnum19NotUsedOnWire
}
register_dcerpc_interface(
    name="TermSrvSession",
    uuid=uuid.UUID("484809d6-4239-471b-b5bc-61df8c23ac48"),
    version="1.0",
    opnums=TERMSRVSESSION_OPNUMS,
)


class RpcWaitForSessionState_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("SessionId", 0),
        NDRSignedIntField("State", 0),
        NDRIntField("Timeout", 0),
    ]


class RpcWaitForSessionState_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcRegisterAsyncNotification_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("SessionId", 0), NDRIntField("Mask", 0)]


class RpcRegisterAsyncNotification_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phNotify", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class SESSION_CHANGE(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRSignedIntField("SessionId", 0), NDRIntField("NotificationId", 0)]


class RpcWaitAsyncNotification_Request(NDRPacket):
    fields_desc = [NDRPacketField("hNotify", NDRContextHandle(), NDRContextHandle)]


class RpcWaitAsyncNotification_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "SessionChange",
            [],
            SESSION_CHANGE,
            size_is=lambda pkt: pkt.pEntries,
            ptr_pack=True,
        ),
        NDRIntField("pEntries", None, size_of="SessionChange"),
        NDRIntField("status", 0),
    ]


class RpcUnRegisterAsyncNotification_Request(NDRPacket):
    fields_desc = [NDRPacketField("phNotify", NDRContextHandle(), NDRContextHandle)]


class RpcUnRegisterAsyncNotification_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phNotify", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


TERMSRVNOTIFICATION_OPNUMS = {
    0: DceRpcOp(RpcWaitForSessionState_Request, RpcWaitForSessionState_Response),
    1: DceRpcOp(
        RpcRegisterAsyncNotification_Request, RpcRegisterAsyncNotification_Response
    ),
    2: DceRpcOp(RpcWaitAsyncNotification_Request, RpcWaitAsyncNotification_Response),
    3: DceRpcOp(
        RpcUnRegisterAsyncNotification_Request, RpcUnRegisterAsyncNotification_Response
    ),
}
register_dcerpc_interface(
    name="TermSrvNotification",
    uuid=uuid.UUID("11899a43-2b68-4a76-92e3-a3d6ad8c26ce"),
    version="1.0",
    opnums=TERMSRVNOTIFICATION_OPNUMS,
)


class RpcOpenEnum_Request(NDRPacket):
    fields_desc = []


class RpcOpenEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phEnum", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RpcCloseEnum_Request(NDRPacket):
    fields_desc = [NDRPacketField("phEnum", NDRContextHandle(), NDRContextHandle)]


class RpcCloseEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phEnum", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RpcFilterByState_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle),
        NDRSignedIntField("State", 0),
        NDRSignedIntField("bInvert", 0),
    ]


class RpcFilterByState_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcFilterByCallersName_Request(NDRPacket):
    fields_desc = [NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle)]


class RpcFilterByCallersName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RpcEnumAddFilter_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("hSubEnum", NDRContextHandle(), NDRContextHandle),
    ]


class RpcEnumAddFilter_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SESSIONENUM_LEVEL1(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("SessionId", 0),
        NDRSignedIntField("State", 0),
        StrFixedLenFieldUtf16("Name", "", length=33 * 2),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class SESSIONENUM_LEVEL2(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("SessionId", 0),
        NDRSignedIntField("State", 0),
        StrFixedLenFieldUtf16("Name", "", length=33 * 2),
        NDRIntField("Source", 0),
        NDRSignedIntField("bFullDesktop", 0),
        NDRPacketField("SessionType", GUID(), GUID),
    ]


class PSESSIONENUM(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField("Data", SESSIONENUM_LEVEL1(), SESSIONENUM_LEVEL1),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRPacketField("Data", SESSIONENUM_LEVEL2(), SESSIONENUM_LEVEL2),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
            ],
            StrFixedLenField("Data", "", length=0),
            align=(4, 4),
            switch_fmt=("L", "L"),
        ),
    ]


class RpcGetEnumResult_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Level", 0),
    ]


class RpcGetEnumResult_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "ppSessionEnumResult",
            [],
            PSESSIONENUM,
            size_is=lambda pkt: pkt.pEntries,
            ptr_pack=True,
        ),
        NDRIntField("pEntries", None, size_of="ppSessionEnumResult"),
        NDRIntField("status", 0),
    ]


class RpcFilterBySessionType_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("pSessionType", GUID(), GUID),
    ]


class RpcFilterBySessionType_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SESSION_FILTER(IntEnum):
    SF_SERVICES_SESSION_POPUP = 1


class RpcGetSessionIds_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("Filter", 0, SESSION_FILTER),
        NDRIntField("MaxEntries", 0),
    ]


class RpcGetSessionIds_Response(NDRPacket):
    fields_desc = [
        NDRConfFieldListField(
            "pSessionIds",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pcSessionIds,
            ptr_pack=True,
        ),
        NDRIntField("pcSessionIds", None, size_of="pSessionIds"),
        NDRIntField("status", 0),
    ]


class SESSIONENUM_LEVEL3(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("SessionId", 0),
        NDRSignedIntField("State", 0),
        StrFixedLenFieldUtf16("Name", "", length=33 * 2),
        NDRIntField("Source", 0),
        NDRSignedIntField("bFullDesktop", 0),
        NDRPacketField("SessionType", GUID(), GUID),
        NDRIntField("ProtoDataSize", None, size_of="pProtocolData"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pProtocolData", "", size_is=lambda pkt: pkt.ProtoDataSize
            ),
            deferred=True,
        ),
    ]


class PSESSIONENUM_EX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField("Data", SESSIONENUM_LEVEL1(), SESSIONENUM_LEVEL1),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRPacketField("Data", SESSIONENUM_LEVEL2(), SESSIONENUM_LEVEL2),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRPacketField("Data", SESSIONENUM_LEVEL3(), SESSIONENUM_LEVEL3),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("Data", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class RpcGetEnumResultEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hEnum", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Level", 0),
    ]


class RpcGetEnumResultEx_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "ppSessionEnumResult",
            [],
            PSESSIONENUM_EX,
            size_is=lambda pkt: pkt.pEntries,
            ptr_pack=True,
        ),
        NDRIntField("pEntries", None, size_of="ppSessionEnumResult"),
        NDRIntField("status", 0),
    ]


class EXECENVDATA_LEVEL1(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("ExecEnvId", 0),
        NDRSignedIntField("State", 0),
        StrFixedLenFieldUtf16("SessionName", "", length=33 * 2),
    ]


class EXECENVDATA_LEVEL2(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRSignedIntField("ExecEnvId", 0),
        NDRSignedIntField("State", 0),
        StrFixedLenFieldUtf16("SessionName", "", length=33 * 2),
        NDRSignedIntField("AbsSessionId", 0),
        StrFixedLenFieldUtf16("HostName", "", length=33 * 2),
        StrFixedLenFieldUtf16("UserName", "", length=33 * 2),
        StrFixedLenFieldUtf16("DomainName", "", length=33 * 2),
        StrFixedLenFieldUtf16("FarmName", "", length=33 * 2),
    ]


class PEXECENVDATA(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField("Data", EXECENVDATA_LEVEL1(), EXECENVDATA_LEVEL1),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRPacketField("Data", EXECENVDATA_LEVEL2(), EXECENVDATA_LEVEL2),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
            ],
            StrFixedLenField("Data", "", length=0),
            align=(4, 4),
            switch_fmt=("L", "L"),
        ),
    ]


class RpcGetAllSessions_Request(NDRPacket):
    fields_desc = [NDRIntField("pLevel", 0)]


class RpcGetAllSessions_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pLevel", 0),
        NDRConfPacketListField(
            "ppSessionData",
            [],
            PEXECENVDATA,
            size_is=lambda pkt: pkt.pcEntries,
            ptr_pack=True,
        ),
        NDRIntField("pcEntries", None, size_of="ppSessionData"),
        NDRIntField("status", 0),
    ]


class EXECENVDATAEX_LEVEL1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("ExecEnvId", 0),
        NDRSignedIntField("State", 0),
        NDRSignedIntField("AbsSessionId", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszSessionName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszHostName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszUserName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszDomainName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pszFarmName", ""), deferred=True
        ),
    ]


class PEXECENVDATAEX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "Data", EXECENVDATAEX_LEVEL1(), EXECENVDATAEX_LEVEL1
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                )
            ],
            StrFixedLenField("Data", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class RpcGetAllSessionsEx_Request(NDRPacket):
    fields_desc = [NDRIntField("Level", 0)]


class RpcGetAllSessionsEx_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "ppSessionData",
            [],
            PEXECENVDATAEX,
            size_is=lambda pkt: pkt.pcEntries,
            ptr_pack=True,
        ),
        NDRIntField("pcEntries", None, size_of="ppSessionData"),
        NDRIntField("status", 0),
    ]


TERMSRVENUMERATION_OPNUMS = {
    0: DceRpcOp(RpcOpenEnum_Request, RpcOpenEnum_Response),
    1: DceRpcOp(RpcCloseEnum_Request, RpcCloseEnum_Response),
    2: DceRpcOp(RpcFilterByState_Request, RpcFilterByState_Response),
    3: DceRpcOp(RpcFilterByCallersName_Request, RpcFilterByCallersName_Response),
    4: DceRpcOp(RpcEnumAddFilter_Request, RpcEnumAddFilter_Response),
    5: DceRpcOp(RpcGetEnumResult_Request, RpcGetEnumResult_Response),
    6: DceRpcOp(RpcFilterBySessionType_Request, RpcFilterBySessionType_Response),
    # 7: Opnum7NotUsedOnWire,
    8: DceRpcOp(RpcGetSessionIds_Request, RpcGetSessionIds_Response),
    9: DceRpcOp(RpcGetEnumResultEx_Request, RpcGetEnumResultEx_Response),
    10: DceRpcOp(RpcGetAllSessions_Request, RpcGetAllSessions_Response),
    11: DceRpcOp(RpcGetAllSessionsEx_Request, RpcGetAllSessionsEx_Response),
}
register_dcerpc_interface(
    name="TermSrvEnumeration",
    uuid=uuid.UUID("88143fd0-c28d-4b2b-8fef-8d882f6a9390"),
    version="1.0",
    opnums=TERMSRVENUMERATION_OPNUMS,
)



### ms-mqmr.idl
"""
RPC definitions for the following interfaces:
- qmmgmt (v1.0): 41208ee0-e970-11d1-9b9e-00e02c064c39
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRRecursiveField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_dcerpc_interface,
)


class MgmtObjectType(IntEnum):
    MGMT_MACHINE = 1
    MGMT_QUEUE = 2
    MGMT_SESSION = 3


class QUEUE_FORMAT_TYPE(IntEnum):
    QUEUE_FORMAT_TYPE_UNKNOWN = 0
    QUEUE_FORMAT_TYPE_PUBLIC = 1
    QUEUE_FORMAT_TYPE_PRIVATE = 2
    QUEUE_FORMAT_TYPE_DIRECT = 3
    QUEUE_FORMAT_TYPE_MACHINE = 4
    QUEUE_FORMAT_TYPE_CONNECTOR = 5
    QUEUE_FORMAT_TYPE_DL = 6
    QUEUE_FORMAT_TYPE_MULTICAST = 7
    QUEUE_FORMAT_TYPE_SUBQUEUE = 8


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class OBJECTID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Lineage", GUID(), GUID),
        NDRIntField("Uniquifier", 0),
    ]


class DL_ID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("m_DlGuid", GUID(), GUID),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("m_pwzDomain", ""), deferred=True
        ),
    ]


class MULTICAST_ID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("m_address", 0), NDRIntField("m_port", 0)]


class QUEUE_FORMAT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("m_qft", 0),
        NDRByteField("m_SuffixAndFlags", 0),
        NDRShortField("m_reserved", 0),
        NDRUnionField(
            [
                (
                    StrFixedLenField("value", "", length=0),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", OBJECTID(), OBJECTID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", DL_ID(), DL_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", MULTICAST_ID(), MULTICAST_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(1, 8),
            switch_fmt=("B", "B"),
        ),
    ]


class MGMT_OBJECT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRInt3264EnumField("type", 0, MgmtObjectType),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField("value", QUEUE_FORMAT(), QUEUE_FORMAT),
                        deferred=True,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "type", None)
                            == MgmtObjectType.MGMT_QUEUE
                        ),
                        (lambda _, val: val.tag == MgmtObjectType.MGMT_QUEUE),
                    ),
                ),
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "type", None)
                            == MgmtObjectType.MGMT_MACHINE
                        ),
                        (lambda _, val: val.tag == MgmtObjectType.MGMT_MACHINE),
                    ),
                ),
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "type", None)
                            == MgmtObjectType.MGMT_SESSION
                        ),
                        (lambda _, val: val.tag == MgmtObjectType.MGMT_SESSION),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(2, 8),
            switch_fmt=("H", "I"),
        ),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_LPWSTR = 31
    VT_BLOB = 65
    VT_CLSID = 72
    VT_VECTOR = 4096


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class ULARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("QuadPart", 0)]


class BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbSize", None, size_of="pBlobData"),
        NDRFullPointerField(
            NDRConfStrLenField("pBlobData", "", size_is=lambda pkt: pkt.cbSize),
            deferred=True,
        ),
    ]


class CAUB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenField("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAUI(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRSignedIntField("pElems", 0),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class CAUL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems", [], NDRIntField("pElems", 0), size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CAUH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pElems", [], ULARGE_INTEGER, size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CACLSID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField("pElems", [], GUID, size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CALPWSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("pElems", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class tag_inner_PROPVARIANT(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("vt", 0),
        NDRByteField("wReserved1", 0),
        NDRByteField("wReserved2", 0),
        NDRIntField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class R_QMMgmtGetInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", MGMT_OBJECT(), MGMT_OBJECT),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class R_QMMgmtGetInfo_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRIntField("status", 0),
    ]


class R_QMMgmtAction_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", MGMT_OBJECT(), MGMT_OBJECT),
        NDRShortField("lpwszAction", 0),
    ]


class R_QMMgmtAction_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


QMMGMT_OPNUMS = {
    0: DceRpcOp(R_QMMgmtGetInfo_Request, R_QMMgmtGetInfo_Response),
    1: DceRpcOp(R_QMMgmtAction_Request, R_QMMgmtAction_Response),
}
register_dcerpc_interface(
    name="qmmgmt",
    uuid=uuid.UUID("41208ee0-e970-11d1-9b9e-00e02c064c39"),
    version="1.0",
    opnums=QMMGMT_OPNUMS,
)



### ept.idl
"""
RPC definitions for the following interfaces:
- ept (v3.0): e1af8308-5d1f-11c9-91a4-08002b14a0fa
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarPacketListField,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRVarStrLenField,
    register_dcerpc_interface,
)


class UUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class twr_p_t(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["tower_octet_string"]
    fields_desc = [
        NDRIntField("tower_length", None, size_of="tower_octet_string"),
        NDRConfStrLenField(
            "tower_octet_string",
            "",
            size_is=lambda pkt: pkt.tower_length,
            conformant_in_struct=True,
        ),
    ]


class ept_entry_t(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("object", UUID(), UUID),
        NDRFullPointerField(NDRPacketField("tower", twr_p_t(), twr_p_t), deferred=True),
        NDRVarStrLenField("annotation", ""),
    ]


class ept_insert_Request(NDRPacket):
    fields_desc = [
        NDRIntField("num_ents", None, size_of="entries"),
        NDRConfPacketListField(
            "entries", [], ept_entry_t, size_is=lambda pkt: pkt.num_ents
        ),
        NDRIntField("replace", 0),
    ]


class ept_insert_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ept_delete_Request(NDRPacket):
    fields_desc = [
        NDRIntField("num_ents", None, size_of="entries"),
        NDRConfPacketListField(
            "entries", [], ept_entry_t, size_is=lambda pkt: pkt.num_ents
        ),
    ]


class ept_delete_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RPC_IF_ID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Uuid", UUID(), UUID),
        NDRShortField("VersMajor", 0),
        NDRShortField("VersMinor", 0),
    ]


class ept_lookup_Request(NDRPacket):
    fields_desc = [
        NDRIntField("inquiry_type", 0),
        NDRFullPointerField(NDRPacketField("object", UUID(), UUID)),
        NDRFullPointerField(NDRPacketField("Ifid", RPC_IF_ID(), RPC_IF_ID)),
        NDRIntField("vers_option", 0),
        NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("max_ents", 0),
    ]


class ept_lookup_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("num_ents", None, size_of="entries"),
        NDRConfVarPacketListField(
            "entries",
            [],
            ept_entry_t,
            size_is=lambda pkt: pkt.max_ents,
            length_is=lambda pkt: pkt.num_ents,
        ),
        NDRIntField("status", 0),
    ]


class ept_map_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("obj", UUID(), UUID)),
        NDRFullPointerField(NDRPacketField("map_tower", twr_p_t(), twr_p_t)),
        NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("max_towers", 0),
    ]


class ept_map_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("num_towers", None, size_of="ITowers"),
        NDRConfVarPacketListField(
            "ITowers",
            [],
            twr_p_t,
            size_is=lambda pkt: pkt.max_towers,
            length_is=lambda pkt: pkt.num_towers,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class ept_lookup_handle_free_Request(NDRPacket):
    fields_desc = [NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle)]


class ept_lookup_handle_free_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("entry_handle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class uuid_t(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("time_low", 0),
        NDRShortField("time_mid", 0),
        NDRShortField("time_hi_and_version", 0),
        NDRByteField("clock_seq_hi_and_reserved", 0),
        NDRByteField("clock_seq_low", 0),
        StrFixedLenField("node", "", length=6),
    ]


class ept_inq_object_Request(NDRPacket):
    fields_desc = []


class ept_inq_object_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ept_object", uuid_t(), uuid_t),
        NDRIntField("status", 0),
    ]


class uuid_p_t(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("time_low", 0),
        NDRShortField("time_mid", 0),
        NDRShortField("time_hi_and_version", 0),
        NDRByteField("clock_seq_hi_and_reserved", 0),
        NDRByteField("clock_seq_low", 0),
        StrFixedLenField("node", "", length=6),
    ]


class ept_mgmt_delete_Request(NDRPacket):
    fields_desc = [
        NDRIntField("object_speced", 0),
        NDRPacketField("object", uuid_p_t(), uuid_p_t),
        NDRPacketField("tower", twr_p_t(), twr_p_t),
    ]


class ept_mgmt_delete_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


EPT_OPNUMS = {
    0: DceRpcOp(ept_insert_Request, ept_insert_Response),
    1: DceRpcOp(ept_delete_Request, ept_delete_Response),
    2: DceRpcOp(ept_lookup_Request, ept_lookup_Response),
    3: DceRpcOp(ept_map_Request, ept_map_Response),
    4: DceRpcOp(ept_lookup_handle_free_Request, ept_lookup_handle_free_Response),
    5: DceRpcOp(ept_inq_object_Request, ept_inq_object_Response),
    6: DceRpcOp(ept_mgmt_delete_Request, ept_mgmt_delete_Response),
}
register_dcerpc_interface(
    name="ept",
    uuid=uuid.UUID("e1af8308-5d1f-11c9-91a4-08002b14a0fa"),
    version="3.0",
    opnums=EPT_OPNUMS,
)



### ms-pac.idl
"""
RPC definitions for the following interfaces:
-
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField, StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRFieldListField,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
)


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class PSID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class KERB_SID_AND_ATTRIBUTES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(NDRPacketField("Sid", PSID(), PSID), deferred=True),
            deferred=True,
        ),
        NDRIntField("Attributes", 0),
    ]


class PKERB_SID_AND_ATTRIBUTES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(NDRPacketField("Sid", PSID(), PSID), deferred=True),
            deferred=True,
        ),
        NDRIntField("Attributes", 0),
    ]


class GROUP_MEMBERSHIP(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("RelativeId", 0), NDRIntField("Attributes", 0)]


class PGROUP_MEMBERSHIP(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("RelativeId", 0), NDRIntField("Attributes", 0)]


class PDOMAIN_GROUP_MEMBERSHIP(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("DomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("GroupCount", None, size_of="GroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "GroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.GroupCount,
            ),
            deferred=True,
        ),
    ]


class PDOMAIN_GROUP_MEMBERSHIP(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("DomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("GroupCount", None, size_of="GroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "GroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.GroupCount,
            ),
            deferred=True,
        ),
    ]


class PAC_INFO_BUFFER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("ulType", 0),
        NDRIntField("cbBufferSize", 0),
        NDRLongField("Offset", 0),
    ]


class PACTYPE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("cBuffers", 0),
        NDRIntField("Version", 0),
        PacketListField("Buffers", [], PAC_INFO_BUFFER, count_from=lambda _: 1),
    ]


class PPACTYPE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("cBuffers", 0),
        NDRIntField("Version", 0),
        PacketListField("Buffers", [], PAC_INFO_BUFFER, count_from=lambda _: 1),
    ]


class PPAC_INFO_BUFFER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("ulType", 0),
        NDRIntField("cbBufferSize", 0),
        NDRLongField("Offset", 0),
    ]


class CYPHER_BLOCK(NDRPacket):
    fields_desc = [StrFixedLenField("data", "", length=8)]


class USER_SESSION_KEY(NDRPacket):
    fields_desc = [PacketListField("data", [], CYPHER_BLOCK, count_from=lambda _: 2)]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class RPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class KERB_VALIDATION_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("LogonTime", FILETIME(), FILETIME),
        NDRPacketField("LogoffTime", FILETIME(), FILETIME),
        NDRPacketField("KickOffTime", FILETIME(), FILETIME),
        NDRPacketField("PasswordLastSet", FILETIME(), FILETIME),
        NDRPacketField("PasswordCanChange", FILETIME(), FILETIME),
        NDRPacketField("PasswordMustChange", FILETIME(), FILETIME),
        NDRPacketField("EffectiveName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("FullName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("LogonScript", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("ProfilePath", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("HomeDirectory", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("HomeDirectoryDrive", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRShortField("LogonCount", 0),
        NDRShortField("BadPasswordCount", 0),
        NDRIntField("UserId", 0),
        NDRIntField("PrimaryGroupId", 0),
        NDRIntField("GroupCount", None, size_of="GroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "GroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.GroupCount,
            ),
            deferred=True,
        ),
        NDRIntField("UserFlags", 0),
        NDRPacketField("UserSessionKey", USER_SESSION_KEY(), USER_SESSION_KEY),
        NDRPacketField("LogonServer", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("LogonDomainName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("LogonDomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRFieldListField("Reserved1", [], NDRIntField("", 0), length_is=lambda _: 2),
        NDRIntField("UserAccountControl", 0),
        NDRFieldListField("Reserved3", [], NDRIntField("", 0), length_is=lambda _: 7),
        NDRIntField("SidCount", None, size_of="ExtraSids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ExtraSids",
                [PKERB_SID_AND_ATTRIBUTES()],
                PKERB_SID_AND_ATTRIBUTES,
                size_is=lambda pkt: pkt.SidCount,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ResourceGroupDomainSid", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("ResourceGroupCount", None, size_of="ResourceGroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ResourceGroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.ResourceGroupCount,
            ),
            deferred=True,
        ),
    ]


class PKERB_VALIDATION_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("LogonTime", FILETIME(), FILETIME),
        NDRPacketField("LogoffTime", FILETIME(), FILETIME),
        NDRPacketField("KickOffTime", FILETIME(), FILETIME),
        NDRPacketField("PasswordLastSet", FILETIME(), FILETIME),
        NDRPacketField("PasswordCanChange", FILETIME(), FILETIME),
        NDRPacketField("PasswordMustChange", FILETIME(), FILETIME),
        NDRPacketField("EffectiveName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("FullName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("LogonScript", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("ProfilePath", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("HomeDirectory", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("HomeDirectoryDrive", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRShortField("LogonCount", 0),
        NDRShortField("BadPasswordCount", 0),
        NDRIntField("UserId", 0),
        NDRIntField("PrimaryGroupId", 0),
        NDRIntField("GroupCount", None, size_of="GroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "GroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.GroupCount,
            ),
            deferred=True,
        ),
        NDRIntField("UserFlags", 0),
        NDRPacketField("UserSessionKey", USER_SESSION_KEY(), USER_SESSION_KEY),
        NDRPacketField("LogonServer", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRPacketField("LogonDomainName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("LogonDomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRFieldListField("Reserved1", [], NDRIntField("", 0), length_is=lambda _: 2),
        NDRIntField("UserAccountControl", 0),
        NDRFieldListField("Reserved3", [], NDRIntField("", 0), length_is=lambda _: 7),
        NDRIntField("SidCount", None, size_of="ExtraSids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ExtraSids",
                [PKERB_SID_AND_ATTRIBUTES()],
                PKERB_SID_AND_ATTRIBUTES,
                size_is=lambda pkt: pkt.SidCount,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ResourceGroupDomainSid", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("ResourceGroupCount", None, size_of="ResourceGroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ResourceGroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.ResourceGroupCount,
            ),
            deferred=True,
        ),
    ]


class PAC_CREDENTIAL_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Version", 0),
        NDRIntField("EncryptionType", 0),
        StrFixedLenField("SerializedData", "", length=1),
    ]


class PPAC_CREDENTIAL_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Version", 0),
        NDRIntField("EncryptionType", 0),
        StrFixedLenField("SerializedData", "", length=1),
    ]


class SECPKG_SUPPLEMENTAL_CRED(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("PackageName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRIntField("CredentialSize", None, size_of="Credentials"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "Credentials", "", size_is=lambda pkt: pkt.CredentialSize
            ),
            deferred=True,
        ),
    ]


class PAC_CREDENTIAL_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["Credentials"]
    fields_desc = [
        NDRIntField("CredentialCount", None, size_of="Credentials"),
        NDRConfPacketListField(
            "Credentials",
            [],
            SECPKG_SUPPLEMENTAL_CRED,
            size_is=lambda pkt: pkt.CredentialCount,
            conformant_in_struct=True,
        ),
    ]


class PPAC_CREDENTIAL_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["Credentials"]
    fields_desc = [
        NDRIntField("CredentialCount", None, size_of="Credentials"),
        NDRConfPacketListField(
            "Credentials",
            [],
            SECPKG_SUPPLEMENTAL_CRED,
            size_is=lambda pkt: pkt.CredentialCount,
            conformant_in_struct=True,
        ),
    ]


class PSECPKG_SUPPLEMENTAL_CRED(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("PackageName", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRIntField("CredentialSize", None, size_of="Credentials"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "Credentials", "", size_is=lambda pkt: pkt.CredentialSize
            ),
            deferred=True,
        ),
    ]


class NTLM_SUPPLEMENTAL_CREDENTIAL(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Version", 0),
        NDRIntField("Flags", 0),
        StrFixedLenField("LmPassword", "", length=16),
        StrFixedLenField("NtPassword", "", length=16),
    ]


class PNTLM_SUPPLEMENTAL_CREDENTIAL(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Version", 0),
        NDRIntField("Flags", 0),
        StrFixedLenField("LmPassword", "", length=16),
        StrFixedLenField("NtPassword", "", length=16),
    ]


class PAC_CLIENT_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("ClientId", FILETIME(), FILETIME),
        NDRShortField("NameLength", 0),
        StrFixedLenFieldUtf16("Name", "", length=1 * 2),
    ]


class PPAC_CLIENT_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("ClientId", FILETIME(), FILETIME),
        NDRShortField("NameLength", 0),
        StrFixedLenFieldUtf16("Name", "", length=1 * 2),
    ]


class PRPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class S4U_DELEGATION_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("S4U2proxyTarget", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRIntField("TransitedListSize", None, size_of="S4UTransitedServices"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "S4UTransitedServices",
                [PRPC_UNICODE_STRING()],
                PRPC_UNICODE_STRING,
                size_is=lambda pkt: pkt.TransitedListSize,
            ),
            deferred=True,
        ),
    ]


class PS4U_DELEGATION_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("S4U2proxyTarget", RPC_UNICODE_STRING(), RPC_UNICODE_STRING),
        NDRIntField("TransitedListSize", None, size_of="S4UTransitedServices"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "S4UTransitedServices",
                [PRPC_UNICODE_STRING()],
                PRPC_UNICODE_STRING,
                size_is=lambda pkt: pkt.TransitedListSize,
            ),
            deferred=True,
        ),
    ]


class UPN_DNS_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("UpnLength", 0),
        NDRShortField("UpnOffset", 0),
        NDRShortField("DnsDomainNameLength", 0),
        NDRShortField("DnsDomainNameOffset", 0),
        NDRIntField("Flags", 0),
    ]


class PUPN_DNS_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("UpnLength", 0),
        NDRShortField("UpnOffset", 0),
        NDRShortField("DnsDomainNameLength", 0),
        NDRShortField("DnsDomainNameOffset", 0),
        NDRIntField("Flags", 0),
    ]


class CLAIMS_COMPRESSION_FORMAT(IntEnum):
    COMPRESSION_FORMAT_NONE = 0
    COMPRESSION_FORMAT_LZNT1 = 2
    COMPRESSION_FORMAT_XPRESS = 3
    COMPRESSION_FORMAT_XPRESS_HUFF = 4


class PCLAIMS_SET_METADATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulClaimsSetSize", None, size_of="ClaimsSet"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "ClaimsSet", "", size_is=lambda pkt: pkt.ulClaimsSetSize
            ),
            deferred=True,
        ),
        NDRInt3264EnumField("usCompressionFormat", 0, CLAIMS_COMPRESSION_FORMAT),
        NDRIntField("ulUncompressedClaimsSetSize", 0),
        NDRShortField("usReservedType", 0),
        NDRIntField("ulReservedFieldSize", None, size_of="ReservedField"),
        NDRFullPointerField(
            NDRConfStrLenField(
                "ReservedField", "", size_is=lambda pkt: pkt.ulReservedFieldSize
            ),
            deferred=True,
        ),
    ]


class PAC_CLIENT_CLAIMS_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("Claims", PCLAIMS_SET_METADATA(), PCLAIMS_SET_METADATA),
            deferred=True,
        )
    ]


class PPAC_CLIENT_CLAIMS_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("Claims", PCLAIMS_SET_METADATA(), PCLAIMS_SET_METADATA),
            deferred=True,
        )
    ]


class PAC_DEVICE_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("UserId", 0),
        NDRIntField("PrimaryGroupId", 0),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("AccountDomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("AccountGroupCount", None, size_of="AccountGroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "AccountGroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.AccountGroupCount,
            ),
            deferred=True,
        ),
        NDRIntField("SidCount", None, size_of="ExtraSids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ExtraSids",
                [PKERB_SID_AND_ATTRIBUTES()],
                PKERB_SID_AND_ATTRIBUTES,
                size_is=lambda pkt: pkt.SidCount,
            ),
            deferred=True,
        ),
        NDRIntField("DomainGroupCount", None, size_of="DomainGroup"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "DomainGroup",
                [PDOMAIN_GROUP_MEMBERSHIP()],
                PDOMAIN_GROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.DomainGroupCount,
            ),
            deferred=True,
        ),
    ]


class PPAC_DEVICE_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("UserId", 0),
        NDRIntField("PrimaryGroupId", 0),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("AccountDomainId", PSID(), PSID), deferred=True
            ),
            deferred=True,
        ),
        NDRIntField("AccountGroupCount", None, size_of="AccountGroupIds"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "AccountGroupIds",
                [PGROUP_MEMBERSHIP()],
                PGROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.AccountGroupCount,
            ),
            deferred=True,
        ),
        NDRIntField("SidCount", None, size_of="ExtraSids"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ExtraSids",
                [PKERB_SID_AND_ATTRIBUTES()],
                PKERB_SID_AND_ATTRIBUTES,
                size_is=lambda pkt: pkt.SidCount,
            ),
            deferred=True,
        ),
        NDRIntField("DomainGroupCount", None, size_of="DomainGroup"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "DomainGroup",
                [PDOMAIN_GROUP_MEMBERSHIP()],
                PDOMAIN_GROUP_MEMBERSHIP,
                size_is=lambda pkt: pkt.DomainGroupCount,
            ),
            deferred=True,
        ),
    ]


class PAC_DEVICE_CLAIMS_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("Claims", PCLAIMS_SET_METADATA(), PCLAIMS_SET_METADATA),
            deferred=True,
        )
    ]


class PPAC_DEVICE_CLAIMS_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("Claims", PCLAIMS_SET_METADATA(), PCLAIMS_SET_METADATA),
            deferred=True,
        )
    ]



### ms-tsts_TSVIPRpc.idl
"""
RPC definitions for the following interfaces:
- TSVIPPublic (v1.0): 53b46b02-c73b-4a3e-8dee-b16b80672fc0
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRUnionField,
    NDRVarStrLenField,
    register_dcerpc_interface,
)


class ipv4_sub0(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("sin_port", 0),
        NDRIntField("in_addr", 0),
        StrFixedLenField("sin_zero", "", length=8),
    ]


class ipv6_sub1(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("sin6_port", 0),
        NDRIntField("sin6_flowinfo", 0),
        StrFixedLenFieldUtf16("sin6_addr", "", length=8 * 2),
        NDRIntField("sin6_scope_id", 0),
    ]


class TSVIP_SOCKADDR(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRUnionField(
            [
                (
                    NDRPacketField("u", ipv4_sub0(), ipv4_sub0),
                    ((lambda pkt: None == 2), (lambda _, val: val.tag == 2)),
                ),
                (
                    NDRPacketField("u", ipv6_sub1(), ipv6_sub1),
                    ((lambda pkt: None == 23), (lambda _, val: val.tag == 23)),
                ),
            ],
            StrFixedLenField("u", "", length=0),
            align=(4, 4),
            switch_fmt=("I", "I"),
        )
    ]


class TSVIPAddress(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwVersion", 0),
        NDRPacketField("IPAddress", TSVIP_SOCKADDR(), TSVIP_SOCKADDR),
        NDRIntField("PrefixOrSubnetMask", 0),
        NDRIntField("PhysicalAddressLength", None, size_of="PhysicalAddress"),
        NDRVarStrLenField(
            "PhysicalAddress", "", length_is=lambda pkt: pkt.PhysicalAddressLength
        ),
        NDRIntField("LeaseExpires", 0),
        NDRIntField("T1", 0),
        NDRIntField("T2", 0),
    ]


class TSVIPSession(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwVersion", 0),
        NDRIntField("SessionId", 0),
        NDRPacketField("SessionIP", TSVIPAddress(), TSVIPAddress),
    ]


class RpcGetSessionIP_Request(NDRPacket):
    fields_desc = [NDRShortField("Family", 0), NDRIntField("SessionId", 0)]


class RpcGetSessionIP_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ppVIPSession", TSVIPSession(), TSVIPSession),
        NDRIntField("status", 0),
    ]


TSVIPPUBLIC_OPNUMS = {0: DceRpcOp(RpcGetSessionIP_Request, RpcGetSessionIP_Response)}
register_dcerpc_interface(
    name="TSVIPPublic",
    uuid=uuid.UUID("53b46b02-c73b-4a3e-8dee-b16b80672fc0"),
    version="1.0",
    opnums=TSVIPPUBLIC_OPNUMS,
)
